// Demo 16: CoinTossApplet.java

//      A Frequency Array Demo
//          A Java Applet
//
//      by Charles W. Neville, 1998
//   Copyright Charles W. Neville, 1998

// Portions of this program are modifications of Lewis and Loftus's 
// Flip class (pp. 130 -- 131).

// Consider the following probabilistic experiment:  Toss a pair of coins and
// record the number of heads.  The possible outcomes are 0, 1, 2, as the 
// following table shows (H = Head, T = Tail):
//
//     Result of   Outcome of Experiment
//     Coin Toss     (Number of Heads)
//        HH               2
//        HT               1
//        TH               1
//        TT               0
//
// Now repeat this experiment 1000 times, and record the number of occurrences 
// of each of the three outcomes in a FREQUENCY ARRAY, indexed by the possible
// outcomes.  At the end of the program, the frequency array might look like
// this:
//
//    freqArray  | 243 | 505 | 252 |
//    index i       0     1     2
//
// Now print out the frequency of occurrence of each outcome as a percentage on 
// the Applet Panel.  The results might look like this:
//
//    Number of Heads   Frequency (%)
//         0               24.3
//         1               50.5
//         2               25.2

import java.applet.Applet;
import java.awt.*;
import java.util.*;  // have to import java.util.* because Java needs to
                     // know what class Random is.

public class CoinTossApplet extends Applet {

  Random rand;
  
  // Frequency Array
  int[] freqArray;

  public void init() {

     rand = new Random();
   
     // Arrays are created and sized using the new operator.
     // An array of size 3 is indexed by 0, 1 and 2.
     freqArray = new int[3];

  }

  // We will do our calculations in start.  That way, we can repeat them
  // by restarting the applet.
  public void start() {

     // (re)-initialize freqArray to contain all zeroes, because we are
     // going to be keeping a running total in each array box.
     for( int i = 0; i < 3; i++ ) {

        freqArray[i] = 0;
     }

     // build frequency array (do our calculations)
     buildFreqArray(1000);
  }

  // toss one coin.  0 corresponds to tails, 1 to heads.
  public int tossCoin() {

    return Math.abs(rand.nextInt() ) % 2;
  }

  // toss two coins.  return the number of heads.
  public int toss2Coins() {

    return tossCoin() + tossCoin();
  }

  // build the frequency array (fill the frequency array) by repeating
  // the coin tossing experiment n times, and each time incrementing
  // the appropriate array box
  public void buildFreqArray(int n) {

     int result;

     for(int i = 0; i < n; i++) {

        // toss the two coins
        result = toss2Coins();
        // keep running totals by incrementing appropriate array box
        freqArray[result] = freqArray[result] + 1;
     }
  }

  // all paint does is print the table of outcomes on the Applet Panel
  public void paint(Graphics g) {

     double percentage;

     // print table header
     g.drawString("Outcome" + "    " + "Frequency (%)", 50, 50);

     // print table body
     for(int i = 0; i < 3; i++) {

        percentage = (double) freqArray[i]/1000*100; // remember to cast
                           // at least one thing on right to float
                           // or double so / operator returns a float
                           // or double rather than integer quotient
        g.drawString("  " + i + "            " + "   " + percentage, 
                           75, 50 + 20*i + 20);
     }
  }
}

